' Program: SkyIRDEV.bas
' Version: 1.0
' Date:3 November 2003
'
' PROGRAM COMPILED OK. 1308 Words used
' 32 Variables used in the 16F819 from a possible 256
'
' This program is a quick and dirty decode of the BSkyB 'Open' keyboard in order to use it as an ASCII keyboard
' I originally used a PIC 16F877 chip running at 20MHz in the Proton Mk2 development board.
' I worked out the delay times using a 20 MHz Oscilloscope and also by using Timer1 to measure the pulse widths.
' The delay times for a Sky Remote Control Handset appear to be a bit longer.
' I measured the average 1/2 bit times as:
'
' 	  			  	  	 	  	   KbdDelay microseconds for the Keyboard
'								   RemDelay microseconds for the Remote Control Handset
'
' The pulses do not appear to have a 50:50 duty cycle and I have found that the initial delay I used could vary from
' 1820 microseconds to as much as 2190 microseconds with reliable results. However, the half bit delay time only worked
' when set to KbdDelay microseconds for the keyboard and RemDelay microseconds for the remote control handset. 
' NB: When using the keyboard or remote - beware of pressing the "tv/sat" button as this changes the keyboard protocol
'     from sky digibox to tv and I have not programmed this yet so I ignore the tv codes.
' The program outputs to the Serial Terminal at 9600 baud. It gives ASCII characters from the keyboard or the name of the 
' key if not an ASCII character. Try it. Not the most efficient but it works. Now I'm going to write the tramsmitter. 
' My objective is then to combine this with an RF transmitter/receiver pair and build a remote control for the Sky Digibox
' accessible from anywhere in the house. Any other suggestions for its use would be most welcome.
'
' NB: THIS VERSION OF THE PROGRAM HAS BEEN MODIFIED TO WORK WITH THE PROTON_IR DEVELOPMENT BOARD
'    
'
'		Include "IR_SETUP.INC"		   	   '
 
		Device = 16F819
		Xtal = 4
		SERIAL_BAUD = 9600
		RSOUT_PIN = PortB.6
		RSOUT_MODE = TRUE
		RSOUT_PACE = 1

		Dim Buff0 as Dword SYSTEM		   ' Input buffer for data	(SYSTEM keeps it in BANK 0)
		Dim Buff1 as DWord SYSTEM		   ' Consists of Buff0..Buff4 = 2 x DWord = 64 bits = 8 bytes
		Dim DelayTime as Word			   ' Half bit delay time
		Dim BitCount as Byte	   		   ' Counter
		Dim BitPosn	as Byte				   ' Counter
		Dim Command as Byte				   ' Used to capture and decode the command byte
		Dim Keyboard as Byte			   ' R when Remote Handset is used, K when Open Keyboard is used
        Symbol Signal = PORTB.0     	   ' Input data pin from Infrared Receiver
        Symbol CARRY = STATUS.0			   ' Carry flag
		
		
		Symbol KbdDelay = 420			   ' originally = 430 on Proton Mk2
		Symbol RemDelay = 430              ' originally = 440 on Proton Mk2
        
		
		Delayms 500						   ' Wait for PICmicro to settle down
        INTCON = 0						   ' Clear int-flags, Disable interrupts
		DelayTime = KbdDelay			   ' Start with Open Keyboard timings
		Keyboard = "K"

Main:
	    Delayms 200	  					   ' Slows down the repeat rate
        Clear BitCount
        Clear BitPosn	
        FSR = VARPTR(Buff0)				   ' Set the indirect pointer to the start of the buffer space

Main1:
		While Signal = 1: Wend			   ' Wait until first IR signal (goes low) is received
		Delayus 1935 	  				   ' Debounce - ignoring first 4 1/2 bits which are all logic 1 (low)
										   ' Delay = 215 + (4 x KbdDelay) = 1935  
										   ' Experiment shows anything from 1820 to 2190 gives reliable results
		If Signal = 1 Then Main1		   ' Check for signal, it should be low (logic 1)
		   			   	   				   ' If it's high (logic 0), try again 
		While 1 = 1						   ' Create a loop for all of the 1/2 bits
			Set CARRY					   ' Preset the carry bit
        	If Signal = 1 Then Clrc 	   ' Test Signal
										   ' Input bit is high (logic 0) so change carry
										   ' Now the input bit is in the carry flag
			Rol	INDF					   ' Rotate carry into the buffer
			Inc BitCount 				   ' Increment the bit counter
        	If BitCount > 63 Then Decode   ' See if we've done all 64 1/2 bits
			INC	BitPosn	 
			If BitPosn.3 = 1 Then		   ' See if all 8 bits are filled 
        		Inc	FSR					   ' If Yes, use next buffer byte
				Clear BitPosn 			   ' and reset bit position counter
        	Endif
			Delayus DelayTime			   ' Wait 1/2 bit before testing signal again
        Wend 							   ' Loop back & do the rest of the bits

Decode:
        FSR = VARPTR(Buff0)			  	   ' Reset the indirect pointer to the start of the buffer space
Decode1:	  							   ' Check 1st Byte
		if indf = %11001010 then Decode2   ' Looking for &CA as the first byte of the header
		goto main 				 		   ' It isn't so try again
Decode2:   		 	  	  		   		   ' Check 2nd Byte
		inc FSR	
		if indf = %10010011 then Decode3   ' Looking for &93 as byte 2
		gosub KeyboardCheck		 		   ' Switch between keyboard timing and handset timing
		goto Main	   					   ' Then try again
Decode3:   			 	  		   		   ' Check 3rd Byte
		inc FSR	
		if indf = %01010101 then Decode4   ' Looking for &55 as byte 3
		gosub KeyboardCheck		 		   ' Switch between keyboard timing and handset timing
		goto Main
Decode4:   			 	  		   		   ' Check 4th Byte
		inc FSR	
		if indf = %01010101 then Decode5:  ' Looking for &55 as Byte 4
		gosub KeyboardCheck		 		   ' Switch between keyboard timing and handset timing
		goto Main
Decode5:	 	 	   	   				   ' Check 5th Byte NB: not likely to get timing error this far down the decode
		inc FSR	   			 	  		   ' Next Byte (byte 5)
		if indf <> %01010101 then Main	   ' Looking for &55 as Byte 5
		inc FSR	   			 	  		   ' Jump over bytes 6 and 7 because we want to test byte 8 for all zeroes
		inc FSR							   '
		inc FSR							   ' 
		if indf <> %00000000 then Main	   ' Looking for &00 as Byte 8
		
		   		   			 	  		   ' Now decode Bytes 7 then 6 which contain the command id
		dec FSR		 	   	 	  		   ' Pointing now at byte 7
		Clear Command					   ' Clear  Command - this is where the command bits will be assembled
		Clear BitCount					   ' Zeroise the bit counter
CmdLoop:	
		ror indf	  					   ' Data appears to be Manchester encoded with %01 = 0 and %10 = 1
		ror indf						   ' As we are only interested in the second bit position
		ror Command					   	   ' 2 rors gets rid of the first bit and leaves the second in carry
										   ' which we then ror into Command
		inc BitCount					   ' Do this 4 times in Byte 7 then decrement indirect pointer to be
		if BitCount = 4 then dec fsr	   ' pointing at byte 6.
		if Bitcount < 8 then CmdLoop	   ' When we have done this 8 times we have all 8 bits of the command in Command
		select Command	  		 		   ' Don't you just love this new syntax - Well done Les Johnson
		case 0 to 140		   		   	   ' Codes 0 to 140 are control codes
			 select Command				   
			 case 0 to 9						   
			 	  rsout "<", Keyboard, " ", dec Command, ">"
			 case 12
			 	  rsout "<", Keyboard, " ", "Power>"
			 case 32
			 	  rsout "<", Keyboard, " ", "Channel +>"
			 case 33
			 	  rsout "<", Keyboard, " ", "Channel ->"
			 case 60, 99
			 	  rsout "<", Keyboard, " ", "Text>"
			 case 88, 95
			 	  rsout "<", Keyboard, " ", "Up Arrow>"
			 case 89, 96
			 	  rsout "<", Keyboard, " ", "Down Arrow>"
			 case 90, 93
			 	  rsout "<", Keyboard, " ", "Left Arrow>"
			 case 91, 94
			 	  rsout "<", Keyboard, " ", "Right Arrow>"
			 case 92, 97
			 	  rsout "<", Keyboard, " ", "Select>"
			 case 98, 129
			 	  rsout "<", Keyboard, " ", "Help>"
			 case 100
			 	  rsout "<", Keyboard, " ", "Info>"
			 case 101, 131
			 	  rsout "<", Keyboard, " ", "Back Up>"
			 case 102, 109
			 	  rsout "<", Keyboard, " ", "Red>"
			 case 103, 110
			 	  rsout "<", Keyboard, " ", "Green>"
			 case 104, 111
			 	  rsout "<", Keyboard, " ", "Yellow>"
			 case 105, 112
			 	  rsout "<", Keyboard, " ", "Blue>"
			 case 125
			 	  rsout "<", Keyboard, " ", "Box Office>"
			 	  delayms 500		  	   	   ' this code is sent multiple times so delay suppresses the repeats
			 case 126
			 	  rsout "<", Keyboard, " ", "Services>"
			 	  delayms 500		  	   	   ' this code is sent multiple times so delay suppresses the repeats
			 case 128
			 	  rsout "<", Keyboard, " ", "Sky>"
			 	  delayms 500		  	   	   ' this code is sent multiple times so delay suppresses the repeats
			 case 132
			 	  rsout "<", Keyboard, " ", "TV/Sat>"
			 case 136
			 	  rsout "<", Keyboard, " ", "Home>"
			 case 138
			 	  rsout "<", Keyboard, " ", "End>"
			 case 139
			 	  rsout "<", Keyboard, " ", "Page Up>"
			 case 140
			 	  rsout "<", Keyboard, " ", "Page Down>"
			 case else		 		   	   ' should never happen - but just in case .....
				  rsout "<", Keyboard, " ", "Unknown Code ", dec Command, ">"
			 endselect
		case 141 to 145			   	 	   ' codes 141 to 145 are ASCII ESC, TAB, RETURN, BACKSPACE, SPACE - lread below
			 Command = lread Code141 +  Command - 141
			 rsout Command
		case 150 to 175	 		   	 	   ' codes 150 to 175 are ASCII 'A' to 'Z'
			 Command =  Command - 85	   ' so subtract 85 from  Command to get ASCII code
			 rsout Command
		case 176 to 201 	  		   	   ' codes 176 to 201 are ASCII 'a' to 'z'
			 Command =  Command - 79	   ' so subtract 79 from  Command to get ASCII code
			 rsout Command
		case 203   		  		  		   ' code 203 is the remote handset Info key
			 rsout "<", Keyboard, " ", "Info>"
		case 204   		  		  		   ' code 204 is the common (to both handset & keyboard) TV Guide key
			 rsout "<", Keyboard, " ", "TV Guide>"
			 delayms 500		  	   	   ' this code is sent multiple times so delay suppresses the repeats
		case 209   				  	   	   ' code 209 is the Euro Symbol but I don't know how to generate this so ....
			 rsout "<", Keyboard, " ", "Euro>"
		case 210 to 241	  		   		   ' codes 210 to 241 are a mish mash of special characters - lread from table below
			 Command = lread Code210 +  Command - 210
			 rsout Command
		case 245   		  	 		 	   ' code 245 is the common (to both handset & keyboard) Interactive key
			 rsout "<", Keyboard, " ", "Interactive>"
			 delayms 500		  	   	   ' this code is sent multiple times so delay suppresses the repeats
		case 246 to 255	 		   	 	   ' codes 246 to 255 are keyboard '0' to '9'
			 Command =  Command - 198	   ' so subtract 198 from  Command to get ASCII code
			 rsout Command
		case else   		  		   	   ' should never happen - but just in case....
			 rsout "<", Keyboard, " ", "Unknown Code ", dec Command, ">"
		endselect
		Goto Main						   ' get the next character
		
KeyboardCheck: 							   ' Subroutine to switch between dealing with the keyboard and a handset
		if Keyboard = "K" then	 		   ' If we are timing the keyboard (KbdDelay for each 1/2 bit)
		   DelayTime = RemDelay 		   ' then switch to timing for remote control handset
		   Keyboard = "R"				   ' (RemDelay for each half bit)
		else		    				   ' and vice versa
		   DelayTime = KbdDelay
		   Keyboard = "K"
		endif
		return

Code141:
		LDATA 27  ' ESC
		LDATA 9	  ' TAB
		LDATA 13  ' RETURN
		LDATA 8	  ' BACKSPACE
		LDATA 32  ' SPACE
Code210:
		LDATA 33  ' ! (exclamation mark)
		LDATA 34  ' " (double quote)
		LDATA 35  ' # (hash - actually this should be  (pound sign) but I don't know what the ASCII code for this is
	    LDATA 36  ' $ (dollar)
		LDATA 37  ' % (percent)
		LDATA 94  ' ^ (circumflex)
		LDATA 38  ' & (ampersand)
		LDATA 42  ' * (asterisk)
		LDATA 40  ' ( (left bracket)
		LDATA 41  ' ) (right bracket)
		LDATA 95  ' _ (underscore)
		LDATA 45  ' - (minus)
		LDATA 43  ' + (plus)
		LDATA 61  ' = (equals)
		LDATA 123 ' { (left curly bracket)
		LDATA 125 ' } (right curly bracket)
		LDATA 91  ' [ (left square bracket)
		LDATA 93  ' [ (right square bracket)
		LDATA 58  ' : (colon)
		LDATA 59  ' ; (semicolon)
		LDATA 64  ' @ (at)
		LDATA 39  ' ' (single quote)
		LDATA 126 ' ~ (tilde)
		LDATA 35  ' # (hash)
		LDATA 60  ' < (less than)
		LDATA 62  ' > (more than)
		LDATA 44  ' , (comma)
		LDATA 46  ' . (full stop)
		LDATA 63  ' ? (question)
		LDATA 47  ' / forward slash)
		LDATA 92  ' \ (backslash)
		LDATA 124 ' | (vertical bar)

